
/* Chrysalide - Outil d'analyse de fichiers binaires
 * type.c - équivalent Python du fichier "analysis/type.c"
 *
 * Copyright (C) 2018 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "type.h"


#include <malloc.h>
#include <pygobject.h>


#include <analysis/type.h>


#include "../helpers.h"



/* Décrit le type fourni sous forme de caractères. */
static PyObject *py_data_type_to_str(PyObject *);



/******************************************************************************
*                                                                             *
*  Paramètres  : self = instance d'un type version Python à traiter.          *
*                                                                             *
*  Description : Décrit le type fourni sous forme de caractères.              *
*                                                                             *
*  Retour      : Chaîne de caractère construite pour l'occasion.              *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static PyObject *py_data_type_to_str(PyObject *self)
{
    PyObject *result;                       /* Représentation à retourner  */
    GDataType *type;                        /* Version native de l'objet   */
    char *desc;                             /* Description du type         */

    type = G_DATA_TYPE(pygobject_get(self));

    desc = g_data_type_to_string(type, true);

    result = PyUnicode_FromString(desc);

    free(desc);

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : -                                                            *
*                                                                             *
*  Description : Fournit un accès à une définition de type à diffuser.        *
*                                                                             *
*  Retour      : Définition d'objet pour Python.                              *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

PyTypeObject *get_python_data_type_type(void)
{
    static PyMethodDef py_data_type_methods[] = {
        { NULL }
    };

    static PyGetSetDef py_data_type_getseters[] = {
        { NULL }
    };

    static PyTypeObject py_data_type_type = {

        PyVarObject_HEAD_INIT(NULL, 0)

        .tp_name        = "pychrysalide.analysis.DataType",

        .tp_str         = py_data_type_to_str,

        .tp_flags       = Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE,

        .tp_doc         = "PyChrysalide data type",

        .tp_methods     = py_data_type_methods,
        .tp_getset      = py_data_type_getseters

    };

    return &py_data_type_type;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : module = module dont la définition est à compléter.          *
*                                                                             *
*  Description : Prend en charge l'objet 'pychrysalide.analysis.DataType'.    *
*                                                                             *
*  Retour      : Bilan de l'opération.                                        *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

bool register_python_data_type(PyObject *module)
{
    PyTypeObject *py_data_type_type;        /* Type Python 'DataType'      */
    PyObject *dict;                         /* Dictionnaire du module      */

    py_data_type_type = get_python_data_type_type();

    dict = PyModule_GetDict(module);

    if (!register_class_for_pygobject(dict, G_TYPE_DATA_TYPE, py_data_type_type, &PyGObject_Type))
        return false;

    return true;

}
