
/* Chrysalide - Outil d'analyse de fichiers binaires
 * gtkdisplaypanel.h - prototypes pour l'affichage de contenus de binaire
 *
 * Copyright (C) 2016-2017 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Chrysalide.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef _GTKEXT_DISPLAYPANEL_H
#define _GTKEXT_DISPLAYPANEL_H


#include <glib-object.h>


#include "../analysis/binary.h"



#define GTK_TYPE_DISPLAY_PANEL              (gtk_display_panel_get_type())
#define GTK_DISPLAY_PANEL(obj)              (G_TYPE_CHECK_INSTANCE_CAST((obj), GTK_TYPE_DISPLAY_PANEL, GtkDisplayPanel))
#define GTK_DISPLAY_PANEL_CLASS(klass)      (G_TYPE_CHECK_CLASS_CAST((klass), GTK_TYPE_DISPLAY_PANEL, GtkDisplayPanelClass))
#define GTK_IS_DISPLAY_PANEL(obj)           (G_TYPE_CHECK_INSTANCE_TYPE((obj), GTK_TYPE_DISPLAY_PANEL))
#define GTK_IS_DISPLAY_PANEL_CLASS(klass)   (G_TYPE_CHECK_CLASS_TYPE((klass), GTK_TYPE_DISPLAY_PANEL))
#define GTK_DISPLAY_PANEL_GET_CLASS(obj)    (G_TYPE_INSTANCE_GET_CLASS((obj), GTK_TYPE_DISPLAY_PANEL, GtkDisplayPanelClass))


/* Composant d'affichage générique (instance) */
typedef struct _GtkDisplayPanel GtkDisplayPanel;

/* Composant d'affichage générique (classe) */
typedef struct _GtkDisplayPanelClass GtkDisplayPanelClass;


/* Détermine le type du composant d'affichage générique. */
GType gtk_display_panel_get_type(void);

/* Définit si une bordure est à afficher. */
void gtk_display_panel_show_border(GtkDisplayPanel *, bool);

/* Définit si les adresses doivent apparaître dans le rendu. */
void gtk_display_panel_set_addresses_display(GtkDisplayPanel *, bool);

/* Définit si le code doit apparaître dans le rendu. */
void gtk_display_panel_set_code_display(GtkDisplayPanel *, bool);

/* Fournit le binaire associé à la représentation. */
GLoadedBinary *gtk_display_panel_get_binary(const GtkDisplayPanel *);




/* Adaptation d'une position sur une surface */
typedef enum _ScrollPositionTweak
{
    SPT_RAW,                                /* Aucun ajustement            */
    SPT_TOP,                                /* Le plus haut possible       */
    SPT_CENTER,                             /* Au centre de la surface     */
    SPT_BOTTOM                              /* Le plus bas possible        */

} ScrollPositionTweak;

/* Indique la position courante du curseur. */
const vmpa2t *gtk_display_panel_get_caret_location(const GtkDisplayPanel *);

/* Fournit l'élément actif lié à la position courante. */
GObject *gtk_display_panel_get_active_object(const GtkDisplayPanel *);

/* S'assure qu'une adresse donnée est visible à l'écran. */
void _gtk_display_panel_scroll_to_address(GtkDisplayPanel *, const vmpa2t *, ScrollPositionTweak, bool);

#define gtk_display_panel_scroll_to_address(p, a, t) _gtk_display_panel_scroll_to_address(p, a, t, true)



/* Demande à qui veut répondre un déplacement du curseur. */
void gtk_display_panel_request_move(GtkDisplayPanel *, const vmpa2t *);



#endif  /* _GTKEXT_DISPLAYPANEL_H */
